@@ -9,6 +9,8 @@ import struct
 import sys
 import time
 
+import zlib
+
 try:
     from hashlib import md5, sha1
 except ImportError:  # pragma NO COVER
@@ -18,7 +20,8 @@ except ImportError:  # pragma NO COVER
 from eventlet import semaphore
 from eventlet import wsgi
 from eventlet.green import socket
-from eventlet.support import get_errno, six
+from eventlet.support import get_errno
+import six
 
 # Python 2's utf8 decoding is more lenient than we'd like
 # In order to pass autobahn's testsuite we need stricter validation
@@ -35,6 +38,7 @@ for _mod in ('wsaccel.utf8validator', 'autobahn.utf8validator'):
         break
 
 ACCEPTABLE_CLIENT_ERRORS = set((errno.ECONNRESET, errno.EPIPE))
+DEFAULT_MAX_FRAME_LENGTH = 8 << 20
 
 __all__ = ["WebSocketWSGI", "WebSocket"]
 PROTOCOL_GUID = b'258EAFA5-E914-47DA-95CA-C5AB0DC85B11'
@@ -72,14 +76,20 @@ class WebSocketWSGI(object):
     :class:`WebSocket`.  To close the socket, simply return from the
     function.  Note that the server will log the websocket request at
     the time of closure.
+
+    An optional argument max_frame_length can be given, which will set the
+    maximum incoming *uncompressed* payload length of a frame. By default, this
+    is set to 8MiB. Note that excessive values here might create a DOS attack
+    vector.
     """
 
-    def __init__(self, handler):
+    def __init__(self, handler, max_frame_length=DEFAULT_MAX_FRAME_LENGTH):
         self.handler = handler
         self.protocol_version = None
         self.support_legacy_versions = True
         self.supported_protocols = []
         self.origin_checker = None
+        self.max_frame_length = max_frame_length
 
     @classmethod
     def configured(cls,
@@ -132,10 +142,16 @@ class WebSocketWSGI(object):
         ws._send_closing_frame(True)
         # use this undocumented feature of eventlet.wsgi to ensure that it
         # doesn't barf on the fact that we didn't call start_response
-        return wsgi.ALREADY_HANDLED
+        wsgi.WSGI_LOCAL.already_handled = True
+        return []
 
     def _handle_legacy_request(self, environ):
-        sock = environ['eventlet.input'].get_socket()
+        if 'eventlet.input' in environ:
+            sock = environ['eventlet.input'].get_socket()
+        elif 'gunicorn.socket' in environ:
+            sock = environ['gunicorn.socket']
+        else:
+            raise Exception('No eventlet.input or gunicorn.socket present in environ.')
 
         if 'HTTP_SEC_WEBSOCKET_KEY1' in environ:
             self.protocol_version = 76
@@ -191,8 +207,84 @@ class WebSocketWSGI(object):
         sock.sendall(handshake_reply)
         return WebSocket(sock, environ, self.protocol_version)
 
+    def _parse_extension_header(self, header):
+        if header is None:
+            return None
+        res = {}
+        for ext in header.split(","):
+            parts = ext.split(";")
+            config = {}
+            for part in parts[1:]:
+                key_val = part.split("=")
+                if len(key_val) == 1:
+                    config[key_val[0].strip().lower()] = True
+                else:
+                    config[key_val[0].strip().lower()] = key_val[1].strip().strip('"').lower()
+            res.setdefault(parts[0].strip().lower(), []).append(config)
+        return res
+
+    def _negotiate_permessage_deflate(self, extensions):
+        if not extensions:
+            return None
+        deflate = extensions.get("permessage-deflate")
+        if deflate is None:
+            return None
+        for config in deflate:
+            # We'll evaluate each config in the client's preferred order and pick
+            # the first that we can support.
+            want_config = {
+                # These are bool options, we can support both
+                "server_no_context_takeover": config.get("server_no_context_takeover", False),
+                "client_no_context_takeover": config.get("client_no_context_takeover", False)
+            }
+            # These are either bool OR int options. True means the client can accept a value
+            # for the option, a number means the client wants that specific value.
+            max_wbits = min(zlib.MAX_WBITS, 15)
+            mwb = config.get("server_max_window_bits")
+            if mwb is not None:
+                if mwb is True:
+                    want_config["server_max_window_bits"] = max_wbits
+                else:
+                    want_config["server_max_window_bits"] = \
+                        int(config.get("server_max_window_bits", max_wbits))
+                    if not (8 <= want_config["server_max_window_bits"] <= 15):
+                        continue
+            mwb = config.get("client_max_window_bits")
+            if mwb is not None:
+                if mwb is True:
+                    want_config["client_max_window_bits"] = max_wbits
+                else:
+                    want_config["client_max_window_bits"] = \
+                        int(config.get("client_max_window_bits", max_wbits))
+                    if not (8 <= want_config["client_max_window_bits"] <= 15):
+                        continue
+            return want_config
+        return None
+
+    def _format_extension_header(self, parsed_extensions):
+        if not parsed_extensions:
+            return None
+        parts = []
+        for name, config in parsed_extensions.items():
+            ext_parts = [six.b(name)]
+            for key, value in config.items():
+                if value is False:
+                    pass
+                elif value is True:
+                    ext_parts.append(six.b(key))
+                else:
+                    ext_parts.append(six.b("%s=%s" % (key, str(value))))
+            parts.append(b"; ".join(ext_parts))
+        return b", ".join(parts)
+
     def _handle_hybi_request(self, environ):
-        sock = environ['eventlet.input'].get_socket()
+        if 'eventlet.input' in environ:
+            sock = environ['eventlet.input'].get_socket()
+        elif 'gunicorn.socket' in environ:
+            sock = environ['gunicorn.socket']
+        else:
+            raise Exception('No eventlet.input or gunicorn.socket present in environ.')
+
         hybi_version = environ['HTTP_SEC_WEBSOCKET_VERSION']
         if hybi_version not in ('8', '13', ):
             raise BadRequest(status='426 Upgrade Required',
@@ -215,9 +307,6 @@ class WebSocketWSGI(object):
                 if p in self.supported_protocols:
                     negotiated_protocol = p
                     break
-        # extensions = environ.get('HTTP_SEC_WEBSOCKET_EXTENSIONS', None)
-        # if extensions:
-        #    extensions = [i.strip() for i in extensions.split(',')]
 
         key = environ['HTTP_SEC_WEBSOCKET_KEY']
         response = base64.b64encode(sha1(six.b(key) + PROTOCOL_GUID).digest())
@@ -227,9 +316,23 @@ class WebSocketWSGI(object):
                            b"Sec-WebSocket-Accept: " + response]
         if negotiated_protocol:
             handshake_reply.append(b"Sec-WebSocket-Protocol: " + six.b(negotiated_protocol))
+
+        parsed_extensions = {}
+        extensions = self._parse_extension_header(environ.get("HTTP_SEC_WEBSOCKET_EXTENSIONS"))
+
+        deflate = self._negotiate_permessage_deflate(extensions)
+        if deflate is not None:
+            parsed_extensions["permessage-deflate"] = deflate
+
+        formatted_ext = self._format_extension_header(parsed_extensions)
+        if formatted_ext is not None:
+            handshake_reply.append(b"Sec-WebSocket-Extensions: " + formatted_ext)
+
         sock.sendall(b'\r\n'.join(handshake_reply) + b'\r\n\r\n')
         return RFC6455WebSocket(sock, environ, self.protocol_version,
-                                protocol=negotiated_protocol)
+                                protocol=negotiated_protocol,
+                                extensions=parsed_extensions,
+                                max_frame_length=self.max_frame_length)
 
     def _extract_number(self, value):
         """
@@ -274,6 +377,12 @@ class WebSocket(object):
         :param environ: The wsgi environment
         :param version: The WebSocket spec version to follow (default is 76)
         """
+        self.log = environ.get('wsgi.errors', sys.stderr)
+        self.log_context = 'server={shost}/{spath} client={caddr}:{cport}'.format(
+            shost=environ.get('HTTP_HOST'),
+            spath=environ.get('SCRIPT_NAME', '') + environ.get('PATH_INFO', ''),
+            caddr=environ.get('REMOTE_ADDR'), cport=environ.get('REMOTE_PORT'),
+        )
         self.socket = sock
         self.origin = environ.get('HTTP_ORIGIN')
         self.protocol = environ.get('HTTP_WEBSOCKET_PROTOCOL')
@@ -285,8 +394,7 @@ class WebSocket(object):
         self._msgs = collections.deque()
         self._sendlock = semaphore.Semaphore()
 
-    @staticmethod
-    def _pack_message(message):
+    def _pack_message(self, message):
         """Pack the message inside ``00`` and ``FF``
 
         As per the dataframing section (5.3) for the websocket spec
@@ -377,9 +485,14 @@ class WebSocket(object):
     def close(self):
         """Forcibly close the websocket; generally it is preferable to
         return from the handler method."""
-        self._send_closing_frame()
-        self.socket.shutdown(True)
-        self.socket.close()
+        try:
+            self._send_closing_frame(True)
+            self.socket.shutdown(True)
+        except SocketError as e:
+            if e.errno != errno.ENOTCONN:
+                self.log.write('{ctx} socket shutdown error: {e}'.format(ctx=self.log_context, e=e))
+        finally:
+            self.socket.close()
 
 
 class ConnectionClosedError(Exception):
@@ -398,11 +511,18 @@ class ProtocolError(ValueError):
 
 
 class RFC6455WebSocket(WebSocket):
-    def __init__(self, sock, environ, version=13, protocol=None, client=False):
+    def __init__(self, sock, environ, version=13, protocol=None, client=False, extensions=None,
+                 max_frame_length=DEFAULT_MAX_FRAME_LENGTH):
         super(RFC6455WebSocket, self).__init__(sock, environ, version)
         self.iterator = self._iter_frames()
         self.client = client
         self.protocol = protocol
+        self.extensions = extensions or {}
+
+        self._deflate_enc = None
+        self._deflate_dec = None
+        self.max_frame_length = max_frame_length
+        self._remote_close_data = None
 
     class UTF8Decoder(object):
         def __init__(self):
@@ -425,6 +545,45 @@ class RFC6455WebSocket(WebSocket):
                     raise ValueError('Data is not valid unicode')
             return self.decoder.decode(data, final)
 
+    def _get_permessage_deflate_enc(self):
+        options = self.extensions.get("permessage-deflate")
+        if options is None:
+            return None
+
+        def _make():
+            return zlib.compressobj(zlib.Z_DEFAULT_COMPRESSION, zlib.DEFLATED,
+                                    -options.get("client_max_window_bits" if self.client
+                                                 else "server_max_window_bits",
+                                                 zlib.MAX_WBITS))
+
+        if options.get("client_no_context_takeover" if self.client
+                       else "server_no_context_takeover"):
+            # This option means we have to make a new one every time
+            return _make()
+        else:
+            if self._deflate_enc is None:
+                self._deflate_enc = _make()
+            return self._deflate_enc
+
+    def _get_permessage_deflate_dec(self, rsv1):
+        options = self.extensions.get("permessage-deflate")
+        if options is None or not rsv1:
+            return None
+
+        def _make():
+            return zlib.decompressobj(-options.get("server_max_window_bits" if self.client
+                                                   else "client_max_window_bits",
+                                                   zlib.MAX_WBITS))
+
+        if options.get("server_no_context_takeover" if self.client
+                       else "client_no_context_takeover"):
+            # This option means we have to make a new one every time
+            return _make()
+        else:
+            if self._deflate_dec is None:
+                self._deflate_dec = _make()
+            return self._deflate_dec
+
     def _get_bytes(self, numbytes):
         data = b''
         while len(data) < numbytes:
@@ -435,20 +594,30 @@ class RFC6455WebSocket(WebSocket):
         return data
 
     class Message(object):
-        def __init__(self, opcode, decoder=None):
+        def __init__(self, opcode, max_frame_length, decoder=None, decompressor=None):
             self.decoder = decoder
             self.data = []
             self.finished = False
             self.opcode = opcode
+            self.decompressor = decompressor
+            self.max_frame_length = max_frame_length
 
         def push(self, data, final=False):
-            if self.decoder:
-                data = self.decoder.decode(data, final=final)
             self.finished = final
             self.data.append(data)
 
         def getvalue(self):
-            return ('' if self.decoder else b'').join(self.data)
+            data = b"".join(self.data)
+            if not self.opcode & 8 and self.decompressor:
+                data = self.decompressor.decompress(data + b"\x00\x00\xff\xff", self.max_frame_length)
+                if self.decompressor.unconsumed_tail:
+                    raise FailedConnectionError(
+                        1009,
+                        "Incoming compressed frame exceeds length limit of {} bytes.".format(self.max_frame_length))
+
+            if self.decoder:
+                data = self.decoder.decode(data, self.finished)
+            return data
 
     @staticmethod
     def _apply_mask(data, mask, length=None, offset=0):
@@ -459,6 +628,7 @@ class RFC6455WebSocket(WebSocket):
 
     def _handle_control_frame(self, opcode, data):
         if opcode == 8:  # connection close
+            self._remote_close_data = data
             if not data:
                 status = 1000
             elif len(data) > 1:
@@ -512,16 +682,21 @@ class RFC6455WebSocket(WebSocket):
 
     def _recv_frame(self, message=None):
         recv = self._get_bytes
+
+        # Unpacking the frame described in Section 5.2 of RFC6455
+        # (https://tools.ietf.org/html/rfc6455#section-5.2)
         header = recv(2)
         a, b = struct.unpack('!BB', header)
         finished = a >> 7 == 1
         rsv123 = a >> 4 & 7
+        rsv1 = rsv123 & 4
         if rsv123:
-            # must be zero
-            raise FailedConnectionError(
-                1002,
-                "RSV1, RSV2, RSV3: MUST be 0 unless an extension is"
-                " negotiated that defines meanings for non-zero values.")
+            if rsv1 and "permessage-deflate" not in self.extensions:
+                # must be zero - unless it's compressed then rsv1 is true
+                raise FailedConnectionError(
+                    1002,
+                    "RSV1, RSV2, RSV3: MUST be 0 unless an extension is"
+                    " negotiated that defines meanings for non-zero values.")
         opcode = a & 15
         if opcode not in (0, 1, 2, 8, 9, 0xA):
             raise FailedConnectionError(1002, "Unknown opcode received.")
@@ -553,14 +728,19 @@ class RFC6455WebSocket(WebSocket):
             length = struct.unpack('!H', recv(2))[0]
         elif length == 127:
             length = struct.unpack('!Q', recv(8))[0]
+
+        if length > self.max_frame_length:
+            raise FailedConnectionError(1009, "Incoming frame of {} bytes is above length limit of {} bytes.".format(
+                length, self.max_frame_length))
         if masked:
             mask = struct.unpack('!BBBB', recv(4))
         received = 0
         if not message or opcode & 8:
             decoder = self.UTF8Decoder() if opcode == 1 else None
-            message = self.Message(opcode, decoder=decoder)
+            decompressor = self._get_permessage_deflate_dec(rsv1)
+            message = self.Message(opcode, self.max_frame_length, decoder=decoder, decompressor=decompressor)
         if not length:
-            message.push('', final=finished)
+            message.push(b'', final=finished)
         else:
             while received < length:
                 d = self.socket.recv(length - received)
@@ -577,13 +757,32 @@ class RFC6455WebSocket(WebSocket):
                         1007, "Text data must be valid utf-8")
         return message
 
-    @staticmethod
-    def _pack_message(message, masked=False,
+    def _pack_message(self, message, masked=False,
                       continuation=False, final=True, control_code=None):
         is_text = False
         if isinstance(message, six.text_type):
             message = message.encode('utf-8')
             is_text = True
+
+        compress_bit = 0
+        compressor = self._get_permessage_deflate_enc()
+        # Control frames are identified by opcodes where the most significant
+        # bit of the opcode is 1.  Currently defined opcodes for control frames
+        # include 0x8 (Close), 0x9 (Ping), and 0xA (Pong).  Opcodes 0xB-0xF are
+        # reserved for further control frames yet to be defined.
+        # https://datatracker.ietf.org/doc/html/rfc6455#section-5.5
+        is_control_frame = (control_code or 0) & 8
+        # An endpoint MUST NOT set the "Per-Message Compressed" bit of control
+        # frames and non-first fragments of a data message.  An endpoint
+        # receiving such a frame MUST _Fail the WebSocket Connection_.
+        # https://datatracker.ietf.org/doc/html/rfc7692#section-6.1
+        if message and compressor and not is_control_frame:
+            message = compressor.compress(message)
+            message += compressor.flush(zlib.Z_SYNC_FLUSH)
+            assert message[-4:] == b"\x00\x00\xff\xff"
+            message = message[:-4]
+            compress_bit = 1 << 6
+
         length = len(message)
         if not length:
             # no point masking empty data
@@ -597,7 +796,7 @@ class RFC6455WebSocket(WebSocket):
                 raise ProtocolError('Control frame data too large (>125).')
             header = struct.pack('!B', control_code | 1 << 7)
         else:
-            opcode = 0 if continuation else (1 if is_text else 2)
+            opcode = 0 if continuation else ((1 if is_text else 2) | compress_bit)
             header = struct.pack('!B', opcode | (1 << 7 if final else 0))
         lengthdata = 1 << 7 if masked else 0
         if length > 65535:
@@ -655,6 +854,11 @@ class RFC6455WebSocket(WebSocket):
     def close(self, close_data=None):
         """Forcibly close the websocket; generally it is preferable to
         return from the handler method."""
-        self._send_closing_frame(close_data=close_data)
-        self.socket.shutdown(socket.SHUT_WR)
-        self.socket.close()
+        try:
+            self._send_closing_frame(close_data=close_data, ignore_send_errors=True)
+            self.socket.shutdown(socket.SHUT_WR)
+        except SocketError as e:
+            if e.errno != errno.ENOTCONN:
+                self.log.write('{ctx} socket shutdown error: {e}'.format(ctx=self.log_context, e=e))
+        finally:
+            self.socket.close()
