@@ -3,7 +3,8 @@ from eventlet.green import Queue
 from eventlet.green import threading
 from eventlet.green import time
 
-patcher.inject('test.test_queue',
+patcher.inject(
+    'test.test_queue',
     globals(),
     ('Queue', Queue),
     ('threading', threading),
