@@ -12,6 +12,8 @@ from eventlet.green import urllib
 # stupid test_support messing with our mojo
 import test.test_support
 i_r_e = test.test_support.is_resource_enabled
+
+
 def is_resource_enabled(resource):
     if resource == 'network':
         return True
@@ -19,7 +21,8 @@ def is_resource_enabled(resource):
         return i_r_e(resource)
 test.test_support.is_resource_enabled = is_resource_enabled
 
-patcher.inject('test.test_ssl',
+patcher.inject(
+    'test.test_ssl',
     globals(),
     ('asyncore', asyncore),
     ('BaseHTTPServer', BaseHTTPServer),
@@ -29,18 +32,18 @@ patcher.inject('test.test_ssl',
     ('ssl', ssl),
     ('threading', threading),
     ('urllib', urllib))
-    
-    
-# TODO svn.python.org stopped serving up the cert that these tests expect; 
+
+
+# TODO svn.python.org stopped serving up the cert that these tests expect;
 # presumably they've updated svn trunk but the tests in released versions will
-# probably break forever. This is why you don't write tests that connect to 
+# probably break forever. This is why you don't write tests that connect to
 # external servers.
 NetworkedTests.testConnect = lambda s: None
 NetworkedTests.testFetchServerCert = lambda s: None
 NetworkedTests.test_algorithms = lambda s: None
 
 # these don't pass because nonblocking ssl sockets don't report
-# when the socket is closed uncleanly, per the docstring on 
+# when the socket is closed uncleanly, per the docstring on
 # eventlet.green.GreenSSLSocket
 # *TODO: fix and restore these tests
 ThreadedTests.testProtocolSSL2 = lambda s: None
